@@ -29,6 +29,7 @@
 
 #include <stdarg.h>
 #include <glib/gtypes.h>
+#include <glib/gmacros.h>
 
 /* Suppress warnings when GCC is in -pedantic mode and not -std=c99
  */
@@ -192,16 +193,16 @@ GPrintFunc      g_set_printerr_handler  (GPrintFunc      func);
 
 #ifdef G_DISABLE_ASSERT
 
-#define g_assert(expr)		G_STMT_START{ }G_STMT_END
-#define g_assert_not_reached()	G_STMT_START{ }G_STMT_END
+#define g_assert(expr)		G_STMT_START{ (void)0; }G_STMT_END
+#define g_assert_not_reached()	G_STMT_START{ (void)0; }G_STMT_END
 
 #else /* !G_DISABLE_ASSERT */
 
 #ifdef __GNUC__
 
 #define g_assert(expr)			G_STMT_START{		\
-     if (expr) { } else						\
-       g_log (G_LOG_DOMAIN,					\
+     if G_LIKELY(expr) { } else 				\
+        g_log (G_LOG_DOMAIN,					\
 	      G_LOG_LEVEL_ERROR,				\
 	      "file %s: line %d (%s): assertion failed: (%s)",	\
 	      __FILE__,						\
@@ -242,8 +243,8 @@ GPrintFunc      g_set_printerr_handler  (GPrintFunc      func);
 
 #ifdef G_DISABLE_CHECKS
 
-#define g_return_if_fail(expr)			G_STMT_START{ }G_STMT_END
-#define g_return_val_if_fail(expr,val)		G_STMT_START{ }G_STMT_END
+#define g_return_if_fail(expr)			G_STMT_START{ (void)0; }G_STMT_END
+#define g_return_val_if_fail(expr,val)		G_STMT_START{ (void)0; }G_STMT_END
 #define g_return_if_reached()			G_STMT_START{ return; }G_STMT_END
 #define g_return_val_if_reached(val)		G_STMT_START{ return (val); }G_STMT_END
 
@@ -252,7 +253,7 @@ GPrintFunc      g_set_printerr_handler  (GPrintFunc      func);
 #ifdef __GNUC__
 
 #define g_return_if_fail(expr)		G_STMT_START{			\
-     if (expr) { } else							\
+     if G_LIKELY(expr) { } else       					\
        {								\
 	 g_log (G_LOG_DOMAIN,						\
 		G_LOG_LEVEL_CRITICAL,					\
@@ -265,7 +266,7 @@ GPrintFunc      g_set_printerr_handler  (GPrintFunc      func);
        };				}G_STMT_END
 
 #define g_return_val_if_fail(expr,val)	G_STMT_START{			\
-     if (expr) { } else							\
+     if G_LIKELY(expr) { } else						\
        {								\
 	 g_log (G_LOG_DOMAIN,						\
 		G_LOG_LEVEL_CRITICAL,					\
