@@ -20,10 +20,11 @@
  * Boston, MA 02111-1307, USA.
  */
 
-#include <config.h>
+#include "config.h"
+
+#include <string.h>
 
 #include "glib.h"
-#include <string.h>
 
 #ifdef _
 #warning "FIXME remove gettext hack"
