@@ -18,13 +18,13 @@
  *   Boston, MA 02111-1307, USA.
  */
 
-#include <config.h>
+#include "config.h"
+
+#include <stdlib.h>
 
 #include "glib.h"
 #include "gunibreak.h"
 
-#include <stdlib.h>
-
 
 #define TPROP(Page, Char) \
   ((break_property_table[Page] >= G_UNICODE_MAX_TABLE_INDEX) \
