@@ -17,59 +17,11 @@
  * Boston, MA 02111-1307, USA.
  */
 
-#include "config.h"
-
 #include <stdlib.h>
 
 #include "gtypeplugin.h"
 #include "gtypemodule.h"
 
-
-/**
- * SECTION:gtypemodule
- * @short_description: Type loading modules
- * @see_also:<variablelist>
- * <varlistentry>
- * <term>#GTypePlugin</term>
- * <listitem><para>The abstract type loader interface.</para></listitem>
- * </varlistentry>
- * <varlistentry>
- * <term>#GModule</term>
- * <listitem><para>Portable mechanism for dynamically loaded modules.</para></listitem>
- * </varlistentry>
- * </variablelist>
- * @title: GTypeModule
- *
- * #GTypeModule provides a simple implementation of the #GTypePlugin
- * interface. The model of #GTypeModule is a dynamically loaded module
- * which implements some number of types and interface
- * implementations. When the module is loaded, it registers its types
- * and interfaces using g_type_module_register_type() and
- * g_type_module_add_interface().  As long as any instances of these
- * types and interface implementations are in use, the module is kept
- * loaded. When the types and interfaces are gone, the module may be
- * unloaded. If the types and interfaces become used again, the module
- * will be reloaded. Note that the last unref cannot happen in module
- * code, since that would lead to the caller's code being unloaded before
- * g_object_unref() returns to it.
- *
- * Keeping track of whether the module should be loaded or not is done by
- * using a use count - it starts at zero, and whenever it is greater than
- * zero, the module is loaded. The use count is maintained internally by
- * the type system, but also can be explicitly controlled by
- * g_type_module_use() and g_type_module_unuse(). Typically, when loading
- * a module for the first type, g_type_module_use() will be used to load
- * it so that it can initialize its types. At some later point, when the
- * module no longer needs to be loaded except for the type
- * implementations it contains, g_type_module_unuse() is called.
- *
- * #GTypeModule does not actually provide any implementation of module
- * loading and unloading. To create a particular module type you must
- * derive from #GTypeModule and implement the load and unload functions
- * in #GTypeModuleClass.
- */
-
-
 typedef struct _ModuleTypeInfo ModuleTypeInfo;
 typedef struct _ModuleInterfaceInfo ModuleInterfaceInfo;
 
@@ -108,8 +60,8 @@ g_type_module_dispose (GObject *object)
   
   if (module->type_infos || module->interface_infos)
     {
-      g_warning (G_STRLOC ": unsolicitated invocation of g_object_run_dispose() on GTypeModule");
-
+      g_warning (G_STRLOC ": unsolicitated invocation of g_object_dispose() on GTypeModule");
+	     
       g_object_ref (object);
     }
 
@@ -170,7 +122,7 @@ g_type_module_get_type (void)
         NULL,               /* interface_data */
       };
 
-      type_module_type = g_type_register_static (G_TYPE_OBJECT, g_intern_static_string ("GTypeModule"), &type_module_info, G_TYPE_FLAG_ABSTRACT);
+      type_module_type = g_type_register_static (G_TYPE_OBJECT, "GTypeModule", &type_module_info, G_TYPE_FLAG_ABSTRACT);
 
       g_type_add_interface_static (type_module_type, G_TYPE_TYPE_PLUGIN, &iface_info);
     }
@@ -179,12 +131,12 @@ g_type_module_get_type (void)
 }
 
 /**
- * g_type_module_set_name:
+ * g_type_module_set_name
  * @module: a #GTypeModule.
  * @name: a human-readable name to use in error messages.
  * 
  * Sets the name for a #GTypeModule 
- */
+ **/
 void
 g_type_module_set_name (GTypeModule  *module,
 			const gchar  *name)
@@ -237,12 +189,10 @@ g_type_module_find_interface_info (GTypeModule *module,
  * 
  * Increases the use count of a #GTypeModule by one. If the
  * use count was zero before, the plugin will be loaded.
- * If loading the plugin fails, the use count is reset to 
- * its prior value. 
- * 
- * Returns: %FALSE if the plugin needed to be loaded and
- *  loading the plugin failed.
- */
+ *
+ * Return Value: %FALSE if the plugin needed to be loaded and
+ *               loading the plugin failed.
+ **/
 gboolean
 g_type_module_use (GTypeModule *module)
 {
@@ -268,7 +218,6 @@ g_type_module_use (GTypeModule *module)
 	      g_warning ("plugin '%s' failed to register type '%s'\n",
 			 module->name ? module->name : "(unknown)",
 			 g_type_name (type_info->type));
-	      module->use_count--;
 	      return FALSE;
 	    }
 	  
@@ -282,13 +231,13 @@ g_type_module_use (GTypeModule *module)
 /**
  * g_type_module_unuse:
  * @module: a #GTypeModule
- *
+ * 
  * Decreases the use count of a #GTypeModule by one. If the
  * result is zero, the module will be unloaded. (However, the
  * #GTypeModule will not be freed, and types associated with the
- * #GTypeModule are not unregistered. Once a #GTypeModule is
+ * #GTypeModule are not unregistered. Once a #GTypeModule is 
  * initialized, it must exist forever.)
- */
+ **/
 void
 g_type_module_unuse (GTypeModule *module)
 {
@@ -356,26 +305,22 @@ g_type_module_complete_interface_info (GTypePlugin    *plugin,
 
 /**
  * g_type_module_register_type:
- * @module: a #GTypeModule
- * @parent_type: the type for the parent class
- * @type_name: name for the type
- * @type_info: type information structure
- * @flags: flags field providing details about the type
- *
+ * @module:  a #GTypeModule
+ * @parent_type:    the type for the parent class
+ * @type_name:      name for the type
+ * @type_info:      type information structure
+ * @flags:          flags field providing details about the type           
+ * 
  * Looks up or registers a type that is implemented with a particular
- * type plugin. If a type with name @type_name was previously registered,
+ * type plugin. If a type with name @type_name is already registered,
  * the #GType identifier for the type is returned, otherwise the type
  * is newly registered, and the resulting #GType identifier returned.
  *
- * When reregistering a type (typically because a module is unloaded
- * then reloaded, and reinitialized), @module and @parent_type must
- * be the same as they were previously.
- *
  * As long as any instances of the type exist, the type plugin will
  * not be unloaded.
  *
- * Returns: the new or existing type ID
- */
+ * Return value: the type ID for the class.
+ **/
 GType
 g_type_module_register_type (GTypeModule     *module,
 			     GType            parent_type,
@@ -416,9 +361,6 @@ g_type_module_register_type (GTypeModule     *module,
 		     parent_type_name ? parent_type_name : "(unknown)");
 	  return 0;
 	}
-
-      if (module_type_info->info.value_table)
-	g_free ((GTypeValueTable *) module_type_info->info.value_table);
     }
   else
     {
@@ -434,7 +376,7 @@ g_type_module_register_type (GTypeModule     *module,
   module_type_info->info = *type_info;
   if (type_info->value_table)
     module_type_info->info.value_table = g_memdup (type_info->value_table,
-						   sizeof (GTypeValueTable));
+						   sizeof (type_info->value_table));
 
   return module_type_info->type;
 }
@@ -445,14 +387,14 @@ g_type_module_register_type (GTypeModule     *module,
  * @instance_type: type to which to add the interface.
  * @interface_type: interface type to add
  * @interface_info: type information structure
- *
- * Registers an additional interface for a type, whose interface lives
- * in the given type plugin. If the interface was already registered
- * for the type in this plugin, nothing will be done.
+ * 
+ * Registers an additional interface for a type, whose interface
+ * lives in the given type plugin. If the interface was already registered
+ * for the type in this plugin, nothing will be done. 
  *
  * As long as any instances of the type exist, the type plugin will
  * not be unloaded.
- */
+ **/
 void
 g_type_module_add_interface (GTypeModule          *module,
 			     GType                 instance_type,
@@ -501,81 +443,3 @@ g_type_module_add_interface (GTypeModule          *module,
   module_interface_info->loaded = TRUE;
   module_interface_info->info = *interface_info;
 }
-
-/**
- * g_type_module_register_enum:
- * @module: a #GTypeModule
- * @name: name for the type
- * @const_static_values: an array of #GEnumValue structs for the
- *                       possible enumeration values. The array is
- *                       terminated by a struct with all members being
- *                       0.
- *
- * Looks up or registers an enumeration that is implemented with a particular
- * type plugin. If a type with name @type_name was previously registered,
- * the #GType identifier for the type is returned, otherwise the type
- * is newly registered, and the resulting #GType identifier returned.
- *
- * As long as any instances of the type exist, the type plugin will
- * not be unloaded.
- *
- * Since: 2.6
- *
- * Returns: the new or existing type ID
- */
-GType
-g_type_module_register_enum (GTypeModule      *module,
-                             const gchar      *name,
-                             const GEnumValue *const_static_values)
-{
-  GTypeInfo enum_type_info = { 0, };
-
-  g_return_val_if_fail (G_IS_TYPE_MODULE (module), 0);
-  g_return_val_if_fail (name != NULL, 0);
-  g_return_val_if_fail (const_static_values != NULL, 0);
-
-  g_enum_complete_type_info (G_TYPE_ENUM,
-                             &enum_type_info, const_static_values);
-
-  return g_type_module_register_type (G_TYPE_MODULE (module),
-                                      G_TYPE_ENUM, name, &enum_type_info, 0);
-}
-
-/**
- * g_type_module_register_flags:
- * @module: a #GTypeModule
- * @name: name for the type
- * @const_static_values: an array of #GFlagsValue structs for the
- *                       possible flags values. The array is
- *                       terminated by a struct with all members being
- *                       0.
- *
- * Looks up or registers a flags type that is implemented with a particular
- * type plugin. If a type with name @type_name was previously registered,
- * the #GType identifier for the type is returned, otherwise the type
- * is newly registered, and the resulting #GType identifier returned.
- *
- * As long as any instances of the type exist, the type plugin will
- * not be unloaded.
- *
- * Since: 2.6
- *
- * Returns: the new or existing type ID
- */
-GType
-g_type_module_register_flags (GTypeModule      *module,
-                             const gchar       *name,
-                             const GFlagsValue *const_static_values)
-{
-  GTypeInfo flags_type_info = { 0, };
-
-  g_return_val_if_fail (G_IS_TYPE_MODULE (module), 0);
-  g_return_val_if_fail (name != NULL, 0);
-  g_return_val_if_fail (const_static_values != NULL, 0);
-
-  g_flags_complete_type_info (G_TYPE_FLAGS,
-                             &flags_type_info, const_static_values);
-
-  return g_type_module_register_type (G_TYPE_MODULE (module),
-                                      G_TYPE_FLAGS, name, &flags_type_info, 0);
-}
